import React, { useEffect, useState } from 'react'
import BaseTmp from '@/components/BaseTmp'
import api from '@/services/api/index'
import { PlayCircleTwoTone, PauseCircleTwoTone } from '@ant-design/icons'
import { Row, Tabs, Button, Select, Input, Col, Table, Tooltip, Divider, Switch, message, Popconfirm } from 'antd';
import AudioManage from '@/common/AudioManage';
import { useStateSync } from '@/hooks/index'
import { history } from 'umi';
import { descText } from '@/common/MessageManage';
import { getSectionExcludState } from '@/services/api/section'

const {
  getQuestionBank,
  getQuestionCard,
  useQuestionCard,
  delQuestionCard
} = api.question


/** 表格 */
const QuestionTable = ({
  loading,
  dataSource,
  onChangePagination,
  pagination,
  onChangeSwitch,
  onDel,
  onEdit
}: {
  loading: boolean,
  dataSource: any[],
  pagination: IPagination
  onChangePagination: (current: number) => void,
  onChangeSwitch: (check: boolean, state: any) => void,
  onDel: (record: any) => void,
  onEdit: (record: any) => void
}) => {

  const [ playingVoice, setPlayingVoice ] = useState<string>()

  const playAudio = (voiceUrl: string) => {
    AudioManage.play(voiceUrl)
    setPlayingVoice(voiceUrl)
  }

  const closeAudio = () => {
    AudioManage.pause()
    setPlayingVoice('')
  }

  const columns = [
    {
      title: '题卡',
      dataIndex: 'title',
      key: 'title',
    },
    {
      title: '问题',
      dataIndex: 'label',
      key: 'label',
      ellipsis: {
        showTitle: false,
      },
      render: (label: string) => <Tooltip   placement="topLeft" title={label} > { label }  </Tooltip>
    },
    {
      title: '音频',
      dataIndex: 'voiceUrl',
      key: 'voiceUrl',
      render: ( voiceUrl: string ) => (
        <>
          <span style={{cursor: "pointer"}} >
            {
              playingVoice === voiceUrl ?  
                <PauseCircleTwoTone onClick={closeAudio} style={{fontSize: '30px'}} /> 
                :  
                <PlayCircleTwoTone onClick={() => playAudio(voiceUrl)} style={{fontSize: '30px'}} /> 
            }
          </span>
        </>
      )
    },
    {
      title: '是否启用',
      dataIndex: 'state',
      key: 'state',
      render: (state, record) => 
      <Switch 
        checked={record.state === 1} 
        checkedChildren="已启用" 
        unCheckedChildren="已禁用" 
        onChange={(check) => onChangeSwitch(check, record)} 
      />
    },
    {
      title: '创建时间',
      dataIndex: 'createTime',
      key: 'createTime',
    },
    {
      title: '操作',
      dataIndex: 'action',
      key: 'action',
      render: (action, records: any) => (
        <>
          {/* <a >查看</a>
          <Divider type="vertical" /> */}
          <a onClick={() => onEdit(records) } >编辑</a>
          <Divider type="vertical" />
          <Popconfirm
            title="确定要删除这张题卡吗？"
            onConfirm={ () => onDel(records)}
            okText="确定"
            cancelText="取消"
          >
            <a >删除</a>
          </Popconfirm>
        </>
      )
    }
  ];

  return <Table 
    loading={loading}
    style={{marginTop: '20px'}} 
    dataSource={dataSource}   
    columns={columns} 
    pagination={{
      showSizeChanger: false,
      position: ["bottomRight"],
      total: pagination?.total,
      current: pagination?.current, 
      onChange: onChangePagination
    }}
   />;
}

interface ITabsState {
  id: string,
  label: string
}

interface IQuestionSearch {
  QBanklist: ITabsState[],
  currentTabsKey: string,
  sectionList: API.Section[],
  pagination: IPagination,
  onChangeTabs:  (e: string) => void,
  onChangeSection: (id: string) => void,
  onSearch: () => void,
  addQuestionCard: () => void,
  onInputLabel: (e: InputEvent) => void
}

const useStateList = [
  {label: '启用', id: 1, state: 1},
  {label: '禁用', id: -1, state: -1},
]

/** 搜索 */
const QuestionSearch: React.FC<IQuestionSearch> = ({
  sectionList,
  QBanklist,
  currentTabsKey,
  pagination,
  onChangeTabs,
  onChangeSection,
  onSearch,
  addQuestionCard,
  onInputLabel
}) => {

  return (
    <Row>
      <Col 
        span={24}
      > 
        <Row align="middle" >
          <Col span={1}> <span>题库： </span> </Col>
          <Col span={20}>
            <Row gutter={4} >
              {
                QBanklist && QBanklist.map( qb =>
                  <Col  key={qb.id} style={{marginTop: '10px'}} >
                    <Button
                      type={currentTabsKey === qb.id ? "primary" : "default"} 
                      onClick={() => onChangeTabs(qb.id)}
                    >
                      {qb.label}
                    </Button>
                  </Col>
                )
              }
            </Row>
          </Col>
        </Row>
      </Col>  
      <Col span={24} style={{marginTop: '20px'}} > 
        <Row  align="middle" >
          <Col span={1} ><span>学段： </span></Col>
          <Col span={20} >
            <Row gutter={4} >
              {
                sectionList && sectionList.map( section => 
                  <Col 
                    key={section.id} 
                  >
                    <Button 
                      type={ pagination.semesterId  === section.id ? "primary" : "default"}  
                      onClick={() => onChangeSection(section.id) }
                    >
                      {section.semesterName}
                    </Button>
                  </Col> 
                )
              }
              <Input placeholder='请搜索题卡名称/问题内容' style={{width: '300px'}} onInput={onInputLabel} />
            </Row>
          </Col>
        </Row>
      </Col>
      <Col span={24} style={{marginTop: '24px'}} >
        <Row  justify="space-between" >
          <Col span={3}>  </Col>
          <Col>
            <Button type="primary" onClick={onSearch} style={{marginRight: '20px'}} >搜索</Button>             
            <Button type="primary" onClick={addQuestionCard} > 添加题卡 </Button>
          </Col>
        </Row>
      </Col>
    </Row>
  )
}

interface IQuestionTabs {
  children: React.ReactChild,
  onChangeTabs: (e: string) => void
  tabsList: ITabsState[] 
}

interface IPagination {
  total: number,
  current: number,
  state: number | string,
  semesterId: string
}

const initPagination: IPagination = {
  total: 0,
  current: 1,
  state: '',
  semesterId: ''
}

const QuestionCard: React.FC = () => {

  /**TabsData */
  const [ tabsList, setTabsList ] = useState<ITabsState[]>([])
  
  /**current tabs key */
  const [ currentTabsKey,  setCurrentTabsKey ] = useState<string>('')

  /**分页 */
  const [ pagination, setPgination ] = useStateSync<IPagination>(initPagination)

  const [ loading, setLoading ] = useState<boolean>(false)

  /**数据 */
  const [ dataSource, setDataSource ] = useState<Record<string, any>[]>([])
 
  /**教材数据 */
  const [ sectionList, setSectionList ] = useState<API.Section[]>([])

  const [ label, setLabel ] = useState<String>('')

  useEffect( () => {
    
    _getQuestionBank()
    
    GetSectionExcludState()
    
  }, [])

  useEffect(() => {

    _getQuestionCard()

  }, [pagination.current])

  // currentTabsKey, pagination.semesterId, 
  /**删除题卡  */
   const _delQuestionCard = async (records: any) => {
     const { status, result } = await delQuestionCard(records.id)
     if ( status === 200 ) {
       message.success( descText.delTextSuccess )
       _getQuestionCard()
     }
   }

   /**编辑题卡 --> 携带题卡的cardId 跳转到下一添加题卡页面 */
  const onEdit = (records: any) => {
    history.push(`/QuestionBankManage/opration-card?cardId=${records.id}`)
  }

  /**添加题卡 --> 跳转到添加题卡页面 */
  const addQuestionCard = () => {
    history.push('/QuestionBankManage/opration-card')
  }

  /**上下架状态 */
  const onChangeSwitch = (check: boolean, records: any) => {
    _useQuestionCard(check, records)
  }

  /**search */
  const onSearch = () => _getQuestionCard()

  /**search label  */ 
  const onInputLabel = (e) => setLabel(e.target.value)

  /**query condition book */
  const onChangeSection = (id: string) => {
    
    setPgination({
      ...pagination,
      semesterId: pagination.semesterId === id ? '' : id
    })
  }

  /**分页 */
  const onChangePagination = (current: number) => {
    setPgination({
      ...pagination,
      current: current
    })
  }

  /** rquest 启用或者停用题卡 */
  const _useQuestionCard = async (check: boolean, records: any) => {
    console.log(check, 'check');
    const state = check ? 1 : -1
    
    const { status, result } = await useQuestionCard(records.id , {state: state})
    if ( status === 200 && result ) {
        message.success(state ===  1 ? '启用成功' : '禁用成功')
        const $r = dataSource.map( ds =>  {
          return {
            ...ds,
            state: records.id === ds.id ? state :  ds.state
          }
        })
        setDataSource( $r)
    }
  }

  /**获取教材 */
  const GetSectionExcludState = async () => {
    const { status, result } = await getSectionExcludState()
    if (status === 200) {
      setSectionList(result)
    }
  }

  /**获取题卡 */
  const _getQuestionCard = async (_pagination?: IPagination) => {

    setLoading(true)
    
    const { status, result } = await getQuestionCard(_pagination? { 
      bankId: currentTabsKey,
        ..._pagination
      } : {
      bankId: currentTabsKey,
      page: pagination?.current, 
      pageSize: 10,
      state: pagination.state,
      label,
      semesterId: pagination.semesterId
    })
    setLoading(false)
    if ( status === 200 ) {
      console.log(result.records, 'result');
      setDataSource(result.records)
      setPgination({
        ...pagination,
        total: result.total,
        current: result.current
      })
    }
  }

  const _getQuestionBank = async () => {
    const { status, result } =  await getQuestionBank()
    if (status === 200) {
      setTabsList(result)
    }
  }

  const onChangeTabs = (e: string) => {

    if (e === currentTabsKey) {
      setCurrentTabsKey("")
    } else {
      setCurrentTabsKey(e)
    }
    
    setPgination({
      ...pagination,
      banlId: e === currentTabsKey ? '' : e
    })
  }

  return  <BaseTmp>
            <QuestionSearch 
              QBanklist={tabsList}
              currentTabsKey={currentTabsKey}
              sectionList={sectionList}
              pagination={pagination}
              onChangeTabs={onChangeTabs}
              addQuestionCard={addQuestionCard}
              onSearch={onSearch}
              onInputLabel={onInputLabel}
              onChangeSection={onChangeSection}
            />
            <QuestionTable 
              loading={loading}
              pagination={pagination}
              dataSource={dataSource}
              onDel={_delQuestionCard}
              onEdit={onEdit}
              onChangeSwitch={onChangeSwitch}
              onChangePagination={onChangePagination}
            />
          </BaseTmp>
}

export default QuestionCard


